using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;
using System;
using System.Collections.Generic;

namespace Sammakkopeli
{
    public class Sammakkopeli : PhysicsGame
    {
        Vector suunta;

        PhysicsObject sammakko;
        PhysicsObject kieli;

        Timer hyppyLaskuri;
        Timer tarkastaLiike;
        Timer hyppyViive;
        Timer kieliAjastin;
        Timer liikutaKielta;

        IntMeter karpasLaskuri;

        int sammakkoTippuuKertoja = -10;
        int edellinenSuunta = 1;

        bool onkoHypannut = false;
        bool ajastinKaynnissa = false;
        bool liikutetaankoPelihahmoa = false;
        bool kaytetaankoKielta = false;

        double nopeusX = 0.0;
        double nopeusY = -1000.0;
        double nopeusKerroin = 1.0;

        public override void Begin()
        {
            TeeAlkuValikko();
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
        }


        void TeeAlkuValikko()
        {
            ClearAll();
            MultiSelectWindow alkuValikko = new MultiSelectWindow("Sammakkopeli", 
                "Valitse taso", "Ohjaimet", "Lopeta peli");

            alkuValikko.AddItemHandler(0, TeeTasovalikko);
            alkuValikko.AddItemHandler(1, ShowControlHelp);
            alkuValikko.AddItemHandler(2, Exit);
            alkuValikko.DefaultCancel = -1;
            Add(alkuValikko);
        }

        void TeeValikko()
        {
            Pause();
            MultiSelectWindow valikko = new MultiSelectWindow("Taukovalikko",
                "Jatka peli", "Ohjaimet", "Takaisin alkuvalikkoon", "Lopeta peli");
            valikko.ItemSelected += PainettiinValikonNappia;
            Add(valikko);
        }

        void PainettiinValikonNappia(int valinta)
        {
            switch (valinta)
            {
                case 0:
                    Pause();
                    break;
                case 1:
                    Pause();
                    ShowControlHelp();
                    break;
                case 2:
                    TeeAlkuValikko();
                    break;
                case 3:
                    Exit();
                    break;
            }
        }


        void TeeTasovalikko()
        {
            MultiSelectWindow tasovalikko = new MultiSelectWindow("Valitse taso",
                "Taso 1", "Taso 2", "Taso 3", "Takaisin");
            tasovalikko.ItemSelected += PainettiinTasovalikonNappia;
            Add(tasovalikko);
        }

        void PainettiinTasovalikonNappia(int valinta)
        {
            switch (valinta)
            {
                case 0:
                    AlustaKentta();
                    break;
                case 1:
                    AlustaKentta();
                    break;
                case 2:
                    AlustaKentta();
                    break;
                case 3:
                    TeeAlkuValikko();
                    break;
            }
        }

        /// <summary>
        /// Asetetaan ohjaimet peli varten
        /// </summary>
        void AsetaOhjaimet()
        {
            Keyboard.Listen(Key.Left, ButtonState.Down, AsetaNopeus, "Liikuta pelihahmoa vasemmalle", -1);
            Keyboard.Listen(Key.Left, ButtonState.Released, PysaytaSammakko, null);
            Keyboard.Listen(Key.Left, ButtonState.Pressed, MuutaSammakonKuvaa, null, -1);

            Keyboard.Listen(Key.Right, ButtonState.Down, AsetaNopeus, "Liikuta pelihahmoa oikeallle", 1);
            Keyboard.Listen(Key.Right, ButtonState.Released, PysaytaSammakko, null);
            Keyboard.Listen(Key.Right, ButtonState.Pressed, MuutaSammakonKuvaa, null, 1);

            Keyboard.Listen(Key.Up, ButtonState.Pressed, SammakkoHyppaa, "Hypp", 8);
            Keyboard.Listen(Key.Space, ButtonState.Pressed, KaytaKielta, "Kyt kielt");

            Keyboard.Listen(Key.P, ButtonState.Pressed, TeeValikko, "Valikko");
        }

        /// <summary>
        /// Mritetn pelihahmon nopeus
        /// </summary>
        /// <param name="suunta"></param>
        void AsetaNopeus(int suunta)
        {
            liikutetaankoPelihahmoa = true;
            if (onkoHypannut == true && sammakkoTippuuKertoja < 3)
            {
                nopeusX = suunta * nopeusKerroin * 300.0;
            }
            else nopeusX = 0.0;
            if (ajastinKaynnissa == false && onkoHypannut == false)
            {
                ajastinKaynnissa = true;
                hyppyViive.Start();
            }
        }


        void MuutaSammakonKuvaa(int suunta)
        {
            if (suunta != edellinenSuunta)
            {
                sammakko.MirrorImage();
            }
            edellinenSuunta = suunta;
        }

        /// <summary>
        /// Pysytt pelihahmon x-suunnassa
        /// </summary>
        void PysaytaSammakko()
        {
            nopeusX = 0.0;
            liikutetaankoPelihahmoa = false;
        }

        /// <summary>
        /// Luo peliin kentn ja alustaa pelin
        /// </summary>
        void AlustaKentta()
        {
            Level.BackgroundColor = Color.Aqua;

            ColorTileMap ruudut = ColorTileMap.FromLevelAsset("Kentta1");
            ruudut.SetTileMethod(Color.Gold, LuoTaso);
            ruudut.SetTileMethod(Color.Red, LuoPelihahmo);
            ruudut.SetTileMethod(Color.Orange, LuoHilleri);
            ruudut.SetTileMethod(Color.Black, LuoKarpanen);

            ruudut.Execute(40, 40);

            Camera.Zoom(1.5);
            Camera.Follow(sammakko);


            LuoAjastimet();
            LuoLaskurit();
            AsetaMuuttujat();
            AsetaOhjaimet();
        }

        void AsetaMuuttujat()
        {
            sammakkoTippuuKertoja = -10;
            edellinenSuunta = 1;

            onkoHypannut = false;
            ajastinKaynnissa = false;
            liikutetaankoPelihahmoa = false;
            kaytetaankoKielta = false;

            nopeusX = 0.0;
            nopeusY = -1000.0;
            nopeusKerroin = 1.0;
        }


        /// <summary>
        /// Luo ajastimet pelihahmon liikuttamista varten
        /// </summary>
        void LuoAjastimet()
        {
            hyppyLaskuri = new Timer();
            hyppyLaskuri.Interval = 0.05;
            hyppyLaskuri.Timeout += SammakkoTippuu;

            tarkastaLiike = new Timer();
            tarkastaLiike.Interval = 0.05;
            tarkastaLiike.Timeout += LiikutaPelihahmoa;
            tarkastaLiike.Start();

            liikutaKielta = new Timer();
            liikutaKielta.Interval = 0.001;
            liikutaKielta.Timeout += LiikutaKielta;

            hyppyViive = new Timer();
            hyppyViive.Interval = 0.17;
            hyppyViive.Timeout += HyppyViive;

            kieliAjastin = new Timer();
            kieliAjastin.Interval = 0.80;
            kieliAjastin.Timeout += PoistaKieli;
        }


        void LuoLaskurit()
        {
            karpasLaskuri = new IntMeter(0);
            Label karpasNaytto = new Label();
            karpasNaytto.Title = "Krpset: ";
            karpasNaytto.X = Screen.Left + 100.0;
            karpasNaytto.Y = Screen.Top - 100.0;
            karpasNaytto.BindTo(karpasLaskuri);
            Add(karpasNaytto);
        }

        void HyppyViive()
        {
            hyppyViive.Stop();
            hyppyViive.Reset();
            if (liikutetaankoPelihahmoa == true && onkoHypannut == false)
            {
                SammakkoHyppaa(3);
                nopeusKerroin = 1.5;
            }
            ajastinKaynnissa = false;
        }

        /// <summary>
        /// Luo pelihahmon
        /// </summary>
        /// <param name="paikka">piste, johon pelihahmo luodaan</param>
        /// <param name="leveys">pelihahmon leveys</param>
        /// <param name="korkeus">pelihahmon korkeus</param>
        void LuoPelihahmo(Vector paikka, double leveys, double korkeus)
        {
            sammakko = new PhysicsObject(40.0, 26.0);
            sammakko.Image = LoadImage("Sammakko.png");
            sammakko.Position = paikka;
            sammakko.CanRotate = false;
            Add(sammakko);

            AddCollisionHandler(sammakko, "palikka", KasitteleSammakonTormays);
        }


        /// <summary>
        /// Liikuttaa pelihahmoa vasemmalle ja oikealle
        /// </summary>
        void LiikutaPelihahmoa()
        {
            suunta = new Vector(nopeusX, nopeusY);
            sammakko.Velocity = suunta;
        }

        void LiikutaKielta()
        {
            kieli.Position = sammakko.Position
                + new Vector((sammakko.Width / 2 + kieli.Width / 2) * edellinenSuunta, 0);
        }

        /// <summary>
        /// Hypytt pelihahmoa
        /// </summary>
        /// <param name="korkeus"></param>
        void SammakkoHyppaa(int korkeus)
        {
            if (onkoHypannut == false)
            {
                onkoHypannut = true;
                sammakkoTippuuKertoja = korkeus;
                hyppyLaskuri.Start();
            }
        }

        /// <summary>
        /// asettaa pelihahmon tippumaan
        /// </summary>
        void SammakkoTippuu()
        {
            if (sammakkoTippuuKertoja < -9)
            {
                hyppyLaskuri.Stop();
                hyppyLaskuri.Reset();
                nopeusY = -1000.0;
                return;
            }
            nopeusY = sammakkoTippuuKertoja * 100;
            sammakkoTippuuKertoja -= 1;
        }


        void KaytaKielta()
        {
            if (kaytetaankoKielta == false)
            {
                kieli = new PhysicsObject(80, 10);
                kieli.Shape = Shape.Rectangle;
                kieli.Color = Color.Pink;
                kieli.Tag = "kieli";
                kieli.IgnoresCollisionResponse = true;
                kieli.Velocity = sammakko.Velocity;
                kieli.Position = sammakko.Position + new Vector((sammakko.Width / 2 + kieli.Width / 2) * edellinenSuunta, 0);
                kieli.LifetimeLeft = TimeSpan.FromSeconds(0.45);
                kaytetaankoKielta = true;
                kieliAjastin.Start(1);
                liikutaKielta.Start();
                AddCollisionHandler(kieli, "karpanen", KasitteleKielenTormays);
                Add(kieli);
            }
        }


        void PoistaKieli()
        {
            kaytetaankoKielta = false;
            kieliAjastin.Stop();
            kieliAjastin.Reset();
            liikutaKielta.Stop();
            liikutaKielta.Reset();
        }


        void KasitteleSammakonTormays(PhysicsObject tormaaja, PhysicsObject kohde)
        {
            if (tormaaja.Bottom >= kohde.Top)
            {
                onkoHypannut = false;
                nopeusKerroin = 1.0;
                nopeusY = 0.0;
            }
            if (tormaaja.Top <= kohde.Bottom)
            {
                sammakkoTippuuKertoja = -1;
            }
        }

        void KasitteleKielenTormays(PhysicsObject tormaaja, PhysicsObject kohde)
        {
            kohde.Destroy();
            karpasLaskuri.Value += 1;
        }

        /// <summary>
        /// Luo tason, jonka pll pelihahmo liikkuu
        /// </summary>
        /// <param name="paikka">piste, johon taso luodaan</param>
        /// <param name="leveys">tason leveys</param>
        /// <param name="korkeus">tason korkeus</param>
        void LuoTaso(Vector paikka, double leveys, double korkeus)
        {
            PhysicsObject taso = PhysicsObject.CreateStaticObject(leveys, korkeus);
            taso.Position = paikka;
            taso.Shape = Shape.Rectangle;
            taso.Color = Color.Orange;
            taso.CollisionIgnoreGroup = 1;
            taso.Tag = "palikka";
            Add(taso);
        }


        /// <summary>
        /// Luo peliin vihollisen
        /// </summary>
        /// <param name="paikka">piste, johon vihollinen luodaan</param>
        /// <param name="leveys">vihollisen leveys</param>
        /// <param name="korkeus">vihollisen korkeus</param>
        void LuoHilleri(Vector paikka, double leveys, double korkeus)
        {
            PhysicsObject hilleri = new PhysicsObject(100, 70);
            hilleri.IgnoresCollisionResponse = true;
            hilleri.Position = paikka;
            hilleri.Image = LoadImage("Hilleri.png");
            Add(hilleri);
        }


        /// <summary>
        /// Luo peliin kerttvn krpsen
        /// </summary>
        /// <param name="paikka"></param>
        /// <param name="leveys"></param>
        /// <param name="korkeus"></param>
        void LuoKarpanen(Vector paikka, double leveys, double korkeus)
        {
            PhysicsObject karpanen = PhysicsObject.CreateStaticObject(leveys, korkeus);
            karpanen.Position = paikka;
            karpanen.Image = LoadImage("Karpane.png");
            karpanen.IgnoresCollisionResponse = true;
            karpanen.Tag = "karpanen";
            Add(karpanen);
        }
    }
}